# w.c.s. - web application for online forms
# Copyright (C) 2005-2022  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

from quixote import get_request, get_response, redirect
from quixote.directory import Directory

from wcs.blocks import BlockDef
from wcs.carddef import CardDef
from wcs.categories import Category
from wcs.formdef import FormDef
from wcs.mail_templates import MailTemplate
from wcs.qommon import _, errors, get_cfg, misc, template
from wcs.qommon.afterjobs import AfterJob
from wcs.qommon.backoffice.listing import pagination_links
from wcs.qommon.backoffice.menu import html_top
from wcs.qommon.form import Form, TextWidget
from wcs.qommon.storage import FtsMatch
from wcs.workflows import Workflow


class I18nDirectory(Directory):
    do_not_call_in_templates = True
    _q_exports = ['', 'scan']

    supported_languages = [
        ('en', _('English')),
        ('fr', _('French')),
        ('de', _('German')),
    ]

    def get_enabled_languages(self):
        enabled_languages = get_cfg('language', {}).get('languages') or []
        return [x for x in self.supported_languages if x[0] in enabled_languages]

    def _q_index(self):
        from wcs.i18n import TranslatableMessage

        if TranslatableMessage.count() == 0:
            return self.scan()
        html_top('i18n', title=_('Multilinguism'))
        get_response().breadcrumb.append(('i18n/', _('Multilinguism')))

        supported_languages = [
            x for x in self.get_enabled_languages() if x[0] != get_cfg('language', {}).get('language')
        ]

        criterias = []
        if get_request().form.get('q'):
            criterias.append(FtsMatch(get_request().form.get('q')))

        offset = misc.get_int_or_400(get_request().form.get('offset', 0))
        limit = 25
        total_count = TranslatableMessage.count(criterias)
        context = {
            'has_sidebar': False,
            'q': get_request().form.get('q'),
            'view': self,
            'selected_language': get_request().form.get('lang') or supported_languages[0][0],
            'supported_languages': supported_languages,
            'pagination_links': pagination_links(offset, limit, total_count, load_js=False),
            'messages': TranslatableMessage.select(criterias, offset=offset, limit=limit, order_by='string'),
            'query': get_request().get_query(),
        }

        get_response().add_javascript(['popup.js'])
        return template.QommonTemplateResponse(
            templates=['wcs/backoffice/i18n.html'], context=context, is_django_native=True
        )

    def scan(self):
        job = get_response().add_after_job(
            I18nScanAfterJob(
                label=_('Scanning for translatable text'),
                user_id=get_request().user.id,
                return_url='/backoffice/i18n/',
            )
        )
        job.store()
        return redirect(job.get_processing_url())

    def _q_lookup(self, component):
        if component in [x[0] for x in self.get_enabled_languages()]:
            return LanguageDirectory(component)
        raise errors.TraversalError()


class LanguageDirectory(Directory):
    def __init__(self, lang):
        self.lang = lang

    def _q_lookup(self, component):
        from wcs.i18n import TranslatableMessage

        try:
            msg = TranslatableMessage.get(component)
        except KeyError:
            raise errors.TraversalError()
        return MessageDirectory(self.lang, msg)


class MessageDirectory(Directory):
    _q_exports = ['']

    def __init__(self, lang, msg):
        self.lang = lang
        self.msg = msg

    def _q_index(self):
        form = Form(enctype='multipart/form-data', action='%s' % get_request().get_path_query())
        rows = min(10, max(2, self.msg.string.count('\n')))
        attr = 'string_%s' % self.lang
        form.add(TextWidget, 'translation', value=getattr(self.msg, attr), rows=rows)
        form.add_submit('submit', _('Submit'))
        form.add_submit('cancel', _('Cancel'))
        if form.get_widget('cancel').parse():
            return redirect('../../?' + get_request().get_query())

        if form.is_submitted() and not form.has_errors():
            setattr(self.msg, attr, form.get_widget('translation').parse())
            self.msg.store()
            return redirect('../../?' + get_request().get_query())

        html_top('i18n', title=_('Multilinguism'))
        context = {'html_form': form, 'msg': self.msg}
        return template.QommonTemplateResponse(
            templates=['wcs/backoffice/i18n-message.html'], context=context
        )


class I18nScanAfterJob(AfterJob):
    def done_action_url(self):
        return self.kwargs['return_url']

    def done_action_label(self):
        return _('Go to multilinguism page')

    def done_button_attributes(self):
        return {'data-redirect-auto': 'true'}

    def execute(self):
        from wcs.i18n import TranslatableMessage

        objects = []
        for klass in (FormDef, CardDef, BlockDef, Workflow, MailTemplate, Category):
            objects.extend(klass.select(ignore_errors=True, ignore_migration=True))
        self.total_count = len(objects) * 2  # one for discovery, one for storing
        self.store()

        strings = {(x.context, x.string): x for x in TranslatableMessage.select()}
        for string in strings.values():
            string.locations = []

        for obj in objects:
            if obj is None:
                self.increment_count()
                continue
            for location, context, string in obj.i18n_scan():
                string = string.strip() if string else string
                if not string:
                    continue
                msg = strings.get((context, string))
                if not msg:
                    msg = TranslatableMessage()
                    msg.context = context
                    msg.string = string
                    msg.locations = []
                    strings[(context, string)] = msg
                msg.locations.append(location)
            self.increment_count()

        total_strings = len(strings)
        for i, string in enumerate(strings.values()):
            string.store()
            self.increment_count(len(objects) / total_strings)
